# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "List widget"

import gobject
import clutter

from widget import *
from container import Container
from box import VBox
from image import Image
from scroll_area import ScrollArea
from scrollable import Scrollable

class List(Container, Scrollable):
    """
    A list widget offers an item list where any item can have a combination of 
    text and icon. There is also support for a selector of the current item 
    for non-touch interfaces.
    
    The list widget implements the scrollable interface.
    
    """

    __gsignals__ = {
        'selection-changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, 
                             (gobject.TYPE_PYOBJECT,)),
        'view-changed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_BOOLEAN, 
                        (gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT,))
    }
    
    def __init__(self):
        """ Initialize list """
        
        super(List, self).__init__()
        super(Scrollable, self).__init__()
        
        self._stage = None
        
        self.set_reactive(True)
        self.connect('button-release-event', self.on_release_event)
        self.connect('scroll-event', self.on_scroll_event)
        
        self.items = []
        self._show_selection = None
        self._selected = None
        self._init_elements()
        self._init_animations()
        
        self._update_style(self.style)

    def _init_elements(self):
        """ Initializes graphical elements """
        
        self._background_image = Image()
        self.add(self._background_image)
        
        self._vbox = VBox()
        self.add(self._vbox)
        
        self._selection_image = None
        
    def _init_animations(self):
        """ Initializes animations """
        
        self._timeline = None
        
    def _update_style(self, props=None):
        """ Updates style """
        
        super(List, self)._update_style(props)
        
        for key, value in props:        
            if key == 'background-image':
                self.background_image = value
                self._background_image.set_source(self.background_image)
                self._background_image._update_layout()
            elif key == 'selection-image':
                self.selection_image = value
                if not self._selection_image:
                    self._selection_image = Image()
                    self._vbox.add(self._selection_image)
                self._selection_image.set_source(self.selection_image)
                self._selection_image._update_layout()

    def _update_layout(self):
        """ Updates layout """
        
        super(List, self)._update_layout()
        
        # Subscribe to key-press events
        if not self._stage:
            self._stage = self.get_stage()
            self._stage.connect('key-press-event', self.on_key_press_event) 
        
        # Update list background image and vbox
        self._background_image._update_layout()
        self._vbox._update_layout()
        
        # Update selection image size and position
        if self._selection_image:
            if not self.selected:
                if self._vbox.get_item_natural_height(0) > 0:
                    self._selection_image.natural_height = self._vbox.get_item_natural_height(0)
                    self._selection_image._update_layout()
            else:
                self._selection_image.natural_height = self._vbox.get_item_natural_height(self.selected)
                self._selection_image._update_layout()
                self._selection_image.set_y(self._vbox.get_item(self.selected).get_y())
                
        # Update childs
        for item in self.items:
            item._update_layout()
            
        # Set clip
        self.set_clipu(0, 0, self.get_widthu(), self.get_heightu())
    
    def append(self, item):
        """ Appends an item to the list """
        
        self.items.append(item)
        self._vbox.pack(item)
        if self.selected == None:
            self.selected = 0
        self._update_layout()
        
    def remove(self):
        """ Remove item from list """
        
        self.items.remove(item)
        self._vbox.unpack(item)
        self._update_layout()
    
    def get_show_selection(self):
        """ Retrieve whether selection is to be displayed """
        
        return self._show_selection
        
    def set_show_selection(self, value):
        """ Sets whether selection is to be displayed
        
        value -- (boolean) whether to display selection or not
        """
        
        self._show_selection = value
        
    show_selection = property(get_show_selection, set_show_selection)
    
    def get_selected(self):
        """ Retrieve selected item """
        
        return self._selected
    
    def set_selected(self, item):
        """ Sets item as selected
        
        item -- (widget) a widget
        """
        
        self._selected = item
        self.emit("selection-changed", self._vbox.get_item(item))
        
        # Emit a signal and change selector position case we have one
        if self._selection_image:
            
            # The new allocation box for the reference widget
            new_alloc_boxu = (
              -1,
              self._vbox.get_item(item).get_y(),
              -1,
              self._selection_image.get_heightu()
            )    
            self.emit("view-changed", new_alloc_boxu, 0.1)
        
            # Animation for switching between selected items in list 
            if not self._timeline:
                self._timeline = clutter.Timeline(fps=60, duration=200)
            if self._timeline.is_playing():
                self._timeline.stop()
            self._timeline = clutter.Timeline(fps=60, duration=200)
            alpha = clutter.Alpha(self._timeline, clutter.sine_inc_func)
            self._behaviour = clutter.BehaviourPath(alpha)
            self._behaviour.append_knots(
              (0, self._selection_image.get_y()),
              (0, self._vbox.get_item(item).get_y())
            )
            self._behaviour.apply(self._selection_image)
            self._timeline.start()
    
    selected = property(get_selected, set_selected)
    
    def on_release_event(self, widget, event):
        """ Button release event """
        
        print widget
        
    def on_scroll_event(self, widget, event):
        """ Scroll event """
    
        if event.direction == clutter.SCROLL_UP:
            if self.selected > 0:
                self.selected -= 1
                #self.get_parent().get_parent()._scroll_down(px=self._vbox.get_item(self.selected).get_y())
            else:
                return True
        elif event.direction == clutter.SCROLL_DOWN:
            if self.selected < len(self.items)-1:
                self.selected += 1
                #self.get_parent().get_parent()._scroll_up(px=self._vbox.get_item(self.selected).get_y())
            else:
                return True
        elif event.direction == clutter.SCROLL_LEFT:
            pass
        elif event.direction == clutter.SCROLL_RIGHT:
            pass
            
        return True
            
    def on_key_press_event(self, widget, event):
        """ Key event """
        
        if event.keyval == clutter.keysyms.Up:
            if self.selected > 0:
                self.selected -= 1
        elif event.keyval == clutter.keysyms.Down:
            if self.selected < len(self.items)-1:
                self.selected += 1

