# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Scroll area widget"

import gobject
import clutter

from widget import *
from container import Container
from image import Image
from scrollable import Scrollable

class ScrollArea(Container):
    """
    A scroll area offers a scrollable container area which can be horizontal, 
    vertical, or both. It supports both standard and touch based scrolling.
    
    """
    
    def __init__(self, area_width=1.0, area_height=1.0):
        """ Initialize scroll area """
        
        super(ScrollArea, self).__init__()
        
        # Scroll area dimensions
        self.area_width = area_width
        self.area_height = area_height
        
        self._items = []
        
        # Set actor reactive and subscribe to scroll event
        self.set_reactive(True)
        self.connect('scroll-event', self.do_scroll_event) 
        
        self._init_elements()
        self._init_animations()
        self._update_style(self.style)
        
    def _init_elements(self):
        """ Initializes graphical elements """
        
        self._background_image = Image()
        self.add(self._background_image)
        
        self._scroll_image = Image()
        self.add(self._scroll_image)
        self._scroll_image.natural_width = self.area_width
        self._scroll_image.natural_height = self.area_height
    
    def _init_animations(self):
        """ Initializes animations """
    
        self._timeline = None
        self._horiz_factor=0.0
        self._vert_factor=0.0

    def _update_style(self, props=None):
        """ Updates style """
        
        super(ScrollArea, self)._update_style(props)
        
        for key, value in props:
            if key == 'background-image':
                self.background_image = value
                self._background_image.set_source(self.background_image)
                self._background_image._update_layout()
            elif key == 'scroll-image':
                self.scroll_image = value
                self._scroll_image.set_source(self.scroll_image)
                self._scroll_image._update_layout()   

    def _update_layout(self):
        """ Updates layout """
        
        super(ScrollArea, self)._update_layout()

        # Subscribe to key-press events
        if self.get_stage():
            _stage = self.get_stage()
            _stage.connect('key-press-event', self.do_key_press_event)
            
        # Update background layout
        self._background_image._update_layout()
        
        # Update scroll area layout, account for previous scroll position
        x_factor = y_factor = 0.0
        if self._scroll_image.get_xu() != 0:
            x_factor = \
              self._scroll_image.get_widthu() / self._scroll_image.get_xu()
        if self._scroll_image.get_yu() != 0:
            y_factor = \
              self._scroll_image.get_heightu() / self._scroll_image.get_yu()
        
        self._scroll_image._update_layout()  
        
        if x_factor != 0:
            self._scroll_image.set_xu(
              self._scroll_image.get_widthu() / x_factor
            )
        if y_factor != 0:
            self._scroll_image.set_yu(
              self._scroll_image.get_heightu() / y_factor
            )
        
        # Update layout for packed items
        for item in self._items:
            item._update_layout()
            # FIXME: Raising item to top shouldn't be necessary  
            item.raise_top()

        # Clip the scroll area
        self.set_clipu(0, 0, self.get_widthu(), self.get_heightu())

    def pack(self, widget):
        """ Packs a widget into the scroll area """
        
        self._scroll_image.add(widget)  
        self._items.append(widget)
            
        self._update_layout()

    def unpack(self, widget):
        """ Unpacks a widget from the scroll widget """
        
        self._scroll_image.remove(widget)  
        self._items.remove(widget)
        
        self._update_layout()

    def scroll(self, horiz_factor=0.0, vert_factor=0.0):
        """ Scroll by a horizontal and/or vertical factor 
        
        horiz_factor -- (float) Horizontal factor to scroll
        vert_factor -- (float) Vertical factor to scroll
        """
        
        self._horiz_factor += horiz_factor
        self._vert_factor += vert_factor
        
        # Determine target x and y values for the scroll image
        horiz_factor_as_units = \
          self._horiz_factor * self._scroll_image.get_widthu()
        target_x = self._scroll_image.get_xu() + horiz_factor_as_units

        vert_factor_as_units = \
          self._vert_factor * self._scroll_image.get_heightu()
        target_y = self._scroll_image.get_yu() + vert_factor_as_units
        
        # Normalize targets out of bounds
        if target_x > 0:
            target_x = 0
        elif target_x < self.get_widthu() - self._scroll_image.get_widthu():
            target_x = -(self._scroll_image.get_widthu() - self.get_widthu())
        if target_y > 0: 
            target_y = 0
        elif target_y < self.get_heightu() - self._scroll_image.get_heightu():
            target_y = -(self._scroll_image.get_heightu() - self.get_heightu())     
        
        # Neutralize horizontal and/or vertical factors when the scroll area
        # hits bounds
        right_bound = self.get_widthu() - self._scroll_image.get_widthu()
        if self._scroll_image.get_xu() == 0 or \
          self._scroll_image.get_xu() == right_bound:
            self._horiz_factor = 0.0
        right_bound = self.get_heightu() - self._scroll_image.get_heightu()
        if self._scroll_image.get_yu() == 0 or \
          self._scroll_image.get_yu() == right_bound:
            self._vert_factor = 0.0            
        
        # Kick-start scroll animations
        if self._timeline:
            self._timeline.rewind()
        else:
            self._timeline = clutter.Timeline(fps=60, duration=200)
        alpha = clutter.Alpha(self._timeline, clutter.sine_inc_func)        
        self._behaviour = clutter.BehaviourPath(alpha)
        self._behaviour.append_knots(
          (self._scroll_image.get_x(), self._scroll_image.get_y()), 
          (clutter.units_to_device(target_x), clutter.units_to_device(target_y))
        )
        self._behaviour.apply(self._scroll_image)
        self._timeline.start()

    def do_scroll_event(self, widget, event):
        """ Handles scroll event """

        if event.direction == clutter.SCROLL_UP:
            self.scroll(vert_factor=0.1)
        elif event.direction == clutter.SCROLL_DOWN:
            self.scroll(vert_factor=-0.1)
        elif event.direction == clutter.SCROLL_LEFT:
            self.scroll(0.1)
        elif event.direction == clutter.SCROLL_RIGHT:
            self.scroll(-0.1)
            
    def do_key_press_event(self, widget, event):
        """ Handles key press event """

        if event.keyval == clutter.keysyms.Up:
            self.scroll(vert_factor=0.1)
        elif event.keyval == clutter.keysyms.Down:
            self.scroll(vert_factor=-0.1)
        elif event.keyval == clutter.keysyms.Left:
            self.scroll(0.1)
        elif event.keyval == clutter.keysyms.Right:
            self.scroll(-0.1)
                           
