# !/usr/bin/python
# -*- coding: utf-8 -*-

# Glitter Toolkit

__authors__ = ["Jan Jokela <janjokela@gmail.com>"]
__licenses__ = ["LICENSE.LGPL"]
__description__ = "Video widget"

import os
import sys
import math

import gst
import gobject
import clutter
import cluttergst

from container import Container

class Video(Container):
    """
    A video widget offers easy and simple video playback backed by GStreamer. 
    Features include seamless playback, support for all GStreamer compatible 
    video streams, basic playback control and support for native, widescreen 
    (16:9), TV (4:3) and "smart" aspect ratios.
    
    """
    
    def __init__(self, source=None, aspect_ratio="native"):
        """ Initialize video widget """
        
        super(Video, self).__init__()
        
        self._source = source
        self._aspect_ratio = aspect_ratio
        
        self._texture = None
        
        self._update_style(self.style)
        
    def _update_style(self, props=None):
        """ Updates style """
        
        super(Video, self)._update_style(props)
        
        pass
        
    def _update_layout(self):
        """ Updates layout """
        
        super(Video, self)._update_layout()
        
        if not self._texture:
            self._set_source()
            
        widthu = self.get_widthu()
        heightu = self.get_heightu()

        # Account for 0 width or height
        if widthu == 0 or heightu == 0:
            return

        width_delta = math.fabs(widthu - widthu * 0.5625) 
        height_delta = math.fabs(heightu - heightu * 1.7777)        
        
        # native
        if self.aspect_ratio == "native":
            self._texture.set_positionu(0.0, 0.0)
            self._texture.set_widthu(widthu)
            self._texture.set_heightu(heightu)
        
        # 16:9 widescreen (720p & 1080p)
        elif self.aspect_ratio == "widescreen":
            if width_delta < height_delta:
                video_height = int(widthu * 0.5625)
                self._texture.set_widthu(widthu)
                self._texture.set_heightu(video_height)
                self._texture.set_xu(0)
                self._texture.set_yu(int((heightu - video_height) / 2.0))
            else:
                video_width = int(heightu * 1.7777)
                self._texture.set_widthu(video_width)
                self._texture.set_heightu(heightu)
                self._texture.set_xu(int((widthu - video_width) / 2.0))
                self._texture.set_yu(0)
        
        # 4:3 tv-like        
        elif self.aspect_ratio == "tv":
            if width_delta < height_delta:
                video_height = int(widthu * 0.75)
                self._texture.set_widthu(widthu)
                self._texture.set_heightu(video_height)
                self._texture.set_xu(0)
                self._texture.set_yu(int((heightu - video_height) / 2.0))
            else:
                video_width = int(heightu * 1.3333)
                self._texture.set_widthu(video_width)
                self._texture.set_heightu(heightu)
                self._texture.set_xu(int((widthu - video_width) / 2.0))
                self._texture.set_yu(0)            
            
        # smart: maximum viewing area without cutting too much        
        elif self.aspect_ratio == "smart":
            if width_delta < height_delta:
                video_height = int(widthu * 0.5625)
                self._texture.set_widthu(widthu)
                self._texture.set_heightu(video_height)
                self._texture.set_xu(0)
                self._texture.set_yu(int((heightu - video_height) / 2.0))
            else:
                video_width = int(heightu * 1.7777)
                self._texture.set_widthu(video_width)
                self._texture.set_heightu(heightu)
                self._texture.set_xu(int((widthu - video_width) / 2.0))
                self._texture.set_yu(0)         

        
    def get_source(self):
        """ Retrieve video source (uri) """
        
        return self._source
        
    def set_source(self, value):
        """ Sets video source
        
        value -- (str) source file uri
        """
        
        self._source = value
        
    source = property(get_source, set_source)
    
    def get_aspect_ratio(self):
        """ Retireve aspect ratio """
        
        return self._aspect_ratio
        
    def set_aspect_ratio(self, value):
        """ Sets aspect ratio
        
        value -- (str) 'native', 'widescreen', 'tv' or 'smart'
        """
        
        self._aspect_ratio = value
        self._update_layout()
        
    aspect_ratio = property(get_aspect_ratio, set_aspect_ratio)
    
    def get_position(self):
        """ Retrieve video position as a factor of 1 """
        
        try:
            position = self.playbin.query_position(gst.FORMAT_TIME, None)[0]
            duration = self.playbin.query_duration(gst.FORMAT_TIME, None)[0]
        except gst.QueryError, ex:
            return 0
        
        position_secs = position / 1000000000.0
        duration_secs = duration / 1000000000.0
        
        return position_secs / duration_secs
    
    def set_position(self, position):
        """ Set video position as a factor of 1 """
        
        try:
            duration = self.playbin.query_duration(gst.FORMAT_TIME, None)[0]
        except gst.QueryError, ex:
            return 0
            
        time_ns = duration * position
        
        self.playbin.seek_simple(gst.FORMAT_TIME, gst.SEEK_FLAG_FLUSH, time_ns)
    
    def get_position_as_timestamp(self, mode="ellapsed"):
        """ Retrieve playback position as a timestamp (ex: -3:07)
        
        mode -- (str) "ellapsed" or "left"
        """
        
        try:
            duration = self.playbin.query_duration(gst.FORMAT_TIME, None)[0]
            position = self.playbin.query_position(gst.FORMAT_TIME, None)[0]
            if mode == "ellapsed":
                return self._time_ns_to_stamp(position)
            else:
                return "-" + self._time_ns_to_stamp(duration - position)
        except gst.QueryError, ex:
            return "0:00"
    
    def play(self):
        """ Play video. Resumes or starts video playback """
        
        if not self.is_playing():
            self._texture.set_playing(True)
    
    def pause(self):
        """ Pause video playback """
        
        if self.is_playing():
            self._texture.set_playing(False)
    
    def stop(self):
        """ Stop video playback """
        
        if self._texture:
            self.remove(self._texture)
            self._texture.set_playing(False)
            self._texture.set_property("position", 0)

    def is_playing(self):
        """ Retrieve wether video is playing """
    
        if self._texture:
            if self._texture.get_playing():
                return True
        return False
        
    def _on_gst_message(self, bus, message):
        """ Messages from our gstreamer bus """
        
        if message.type == gst.MESSAGE_EOS:
            self.stop()
        elif message.type == gst.MESSAGE_ERROR:
            print "Gstreamer exception -- bus: " + str(bus) + ", message: " + \
                  str(message)
     
    def _set_source(self):
        """ Set texture based on given source """
        
        width = self.get_width() 
        height = self.get_height()
        
        # Account for 0 width or height
        if width == 0 or height == 0:
            return
            
        # Account for no size change
        if self._texture and \
          width == self._texture.get_width() and \
          height == self._texture.get_height():
            return

        if os.path.exists(self.source):
            self._texture = cluttergst.VideoTexture()
            self.playbin = self._texture.get_playbin()
            self.bus = self.playbin.get_bus()
            self.bus.add_signal_watch()
            self.bus.connect('message', self._on_gst_message)
            self._texture.show()
            self._texture.set_filename(self.source)
            self.add(self._texture)
            self.play()
        else: 
            raise AttributeError, "source doesn't exist: %s" % self.source  

    def _time_ns_to_stamp(self, time_ns):
        """ Converts time given in nanoseconds to a human readable time 
        stamp (ex: 4:23) 
        
        time_ns -- (int) time in nanoseconds
        """
 
        time_s = time_ns / 1000000000
        time_stamp = ""
        
        if time_s >= 3600:
            hours = time_s / 3600
            time_s -= hours * 3600
            time_stamp = str(hours) + ":"
        if time_s >= 600:
            mins = time_s / 60
            time_s -= mins * 60
            time_stamp += str(mins) + ":"
        elif time_s >= 60:
            mins = time_s / 60
            time_s -= mins * 60
            time_stamp += str(mins) + ":"  
        else:
            time_stamp = time_stamp + "0:"
        
        if time_s > 9:
            time_stamp += str(time_s)
        else:
            time_stamp += "0" + str(time_s)
            
        return time_stamp

